@@ -69,6 +69,7 @@ enum GnomeVFSOpType {
 	GNOME_VFS_OP_LOAD_DIRECTORY,
 	GNOME_VFS_OP_XFER,
 	GNOME_VFS_OP_GET_FILE_INFO,
+	GNOME_VFS_OP_SET_FILE_INFO
 };
 typedef enum GnomeVFSOpType GnomeVFSOpType;
 
@@ -179,7 +180,6 @@ typedef struct {
 	struct {
 		GList *uris; /* GnomeVFSURI* */
 		GnomeVFSFileInfoOptions options;
-		gchar **meta_keys;
 	} request;
 
 	struct {
@@ -187,6 +187,21 @@ typedef struct {
 	} notify;
 } GnomeVFSGetFileInfoOp;
 
+typedef struct {
+	struct {
+		GnomeVFSURI *uri;
+		GnomeVFSFileInfo info;
+		GnomeVFSSetFileInfoMask mask;
+		GnomeVFSFileInfoOptions options;
+	} request;
+
+	struct {
+		GnomeVFSResult set_file_info_result;
+		GnomeVFSResult get_file_info_result;
+		GnomeVFSFileInfo info;
+	} notify;
+} GnomeVFSSetFileInfoOp;
+
 
 
 /* "Complex operations.  */
@@ -195,7 +210,6 @@ struct GnomeVFSLoadDirectoryOp {
 	struct {
 		GnomeVFSURI *uri;
 		GnomeVFSFileInfoOptions options;
-		gchar **meta_keys;
 		GnomeVFSDirectorySortRule *sort_rules;
 		gboolean reverse_order;
 		GnomeVFSDirectoryFilterType filter_type;
@@ -247,6 +261,7 @@ typedef union {
 	GnomeVFSLoadDirectoryOp load_directory;
 	GnomeVFSXferOp xfer;
 	GnomeVFSGetFileInfoOp get_file_info;
+	GnomeVFSSetFileInfoOp set_file_info;
 } GnomeVFSSpecificOp;
 
 typedef struct {
