@@ -152,12 +152,13 @@ G_STMT_START { \
 typedef struct {
 	GnomeVFSResult result;
 	union {
-		GnomeVFSAsyncOpenCallback open;
+		GnomeVFSAsyncCallback simple;
 		GnomeVFSAsyncOpenAsChannelCallback open_as_channel;
 		GnomeVFSAsyncCloseCallback close;
 		GnomeVFSAsyncReadCallback read;
 		GnomeVFSAsyncWriteCallback write;
 		GnomeVFSAsyncGetFileInfoCallback get_file_info;
+		GnomeVFSAsyncSetFileInfoCallback set_file_info;
 		GnomeVFSAsyncDirectoryLoadCallback load_directory;
 	} callback;
 	gpointer callback_data;
@@ -168,12 +169,12 @@ typedef struct {
 } CallbackData;
 
 static gboolean
-report_failure_open_callback (gpointer callback_data)
+report_failure_simple_callback (gpointer callback_data)
 {
 	CallbackData *data;
 
 	data = callback_data;
-	(* data->callback.open) (NULL,
+	(* data->callback.simple) (NULL,
 				 data->result,
 				 data->callback_data);
 	g_free (data);
@@ -182,9 +183,9 @@ report_failure_open_callback (gpointer callback_data)
 }
 
 static void
-report_failure_open (GnomeVFSResult result,
-		     GnomeVFSAsyncOpenCallback callback,
-		     gpointer callback_data)
+report_failure_simple (GnomeVFSResult result,
+		       GnomeVFSAsyncCallback callback,
+		       gpointer callback_data)
 {
 	CallbackData *data;
 
@@ -192,10 +193,10 @@ report_failure_open (GnomeVFSResult result,
 		return;
 
 	data = g_new0 (CallbackData, 1);
-	data->callback.open = callback;
+	data->callback.simple = callback;
 	data->result = result;
 	data->callback_data = callback_data;
-	g_idle_add (report_failure_open_callback, data);
+	g_idle_add (report_failure_simple_callback, data);
 }
 
 void
@@ -215,7 +216,7 @@ gnome_vfs_async_open (GnomeVFSAsyncHandle **handle_return,
 
 	CALL_BACKEND (gnome_vfs_async_open,
 		      (handle_return, text_uri, open_mode, callback, callback_data));
-	report_failure_open (result, callback, callback_data);
+	report_failure_simple (result, callback, callback_data);
 }
 
 void
@@ -235,7 +236,7 @@ gnome_vfs_async_open_uri (GnomeVFSAsyncHandle **handle_return,
 
 	CALL_BACKEND (gnome_vfs_async_open_uri,
 		      (handle_return, uri, open_mode, callback, callback_data));
-	report_failure_open (result, callback, callback_data);
+	report_failure_simple (result, callback, callback_data);
 }
 
 static gboolean
@@ -315,7 +316,7 @@ gnome_vfs_async_create (GnomeVFSAsyncHandle **handle_return,
 	CALL_BACKEND (gnome_vfs_async_create,
 		      (handle_return, text_uri, open_mode, exclusive, perm,
 		       callback, callback_data));
-	report_failure_open (result, callback, callback_data);
+	report_failure_simple (result, callback, callback_data);
 }
 
 void
@@ -360,33 +361,33 @@ gnome_vfs_async_create_uri (GnomeVFSAsyncHandle **handle_return,
 						    gboolean exclusive,
 						    guint perm,
 						    GnomeVFSAsyncOpenCallback callback,
-						    gpointer callback_data) = NULL;
+						    gpointer callback_data);
 	GnomeVFSResult result;
 
 	CALL_BACKEND (gnome_vfs_async_create_uri,
 		      (handle_return, text_uri, open_mode, exclusive, perm,
 		       callback, callback_data));
-	report_failure_open (result, callback, callback_data);
+	report_failure_simple (result, callback, callback_data);
 }
 
 
 void
 gnome_vfs_async_create_symbolic_link (GnomeVFSAsyncHandle **handle_return,
-			     GnomeVFSURI *uri,
-			     const gchar *uri_reference,
-			     GnomeVFSAsyncOpenCallback callback,
-			     gpointer callback_data)
+				      GnomeVFSURI *uri,
+				      const gchar *uri_reference,
+				      GnomeVFSAsyncOpenCallback callback,
+				      gpointer callback_data)
 {
 	static GnomeVFSResult
 		(*real_gnome_vfs_async_create_symbolic_link) (GnomeVFSAsyncHandle **handle_return,
 							      GnomeVFSURI *uri,
 							      const gchar *uri_reference,
 							      GnomeVFSAsyncOpenCallback callback,
-							      gpointer callback_data) = NULL;
+							      gpointer callback_data);
 	GnomeVFSResult result;
 	CALL_BACKEND (gnome_vfs_async_create_symbolic_link,
 		      (handle_return, uri, uri_reference, callback, callback_data));
-	report_failure_open (result, callback, callback_data);
+	report_failure_simple (result, callback, callback_data);
 }
 
 static gboolean
@@ -430,7 +431,7 @@ gnome_vfs_async_close (GnomeVFSAsyncHandle *handle,
 	static GnomeVFSResult	 
 		(*real_gnome_vfs_async_close) (GnomeVFSAsyncHandle *handle,
 					       GnomeVFSAsyncCloseCallback callback,
-					       gpointer callback_data) = NULL;
+					       gpointer callback_data);
 	GnomeVFSResult result;
 
 	CALL_BACKEND (gnome_vfs_async_close,
@@ -490,7 +491,7 @@ gnome_vfs_async_read (GnomeVFSAsyncHandle *handle,
 					      gpointer buffer,
 					      guint bytes,
 					      GnomeVFSAsyncReadCallback callback,
-					      gpointer callback_data) = NULL;
+					      gpointer callback_data);
 	GnomeVFSResult result;
 
 	CALL_BACKEND (gnome_vfs_async_read,
@@ -553,7 +554,7 @@ gnome_vfs_async_write (GnomeVFSAsyncHandle *handle,
 					       gconstpointer buffer,
 					       guint bytes,
 					       GnomeVFSAsyncWriteCallback callback,
-					       gpointer callback_data) = NULL;
+					       gpointer callback_data);
 	GnomeVFSResult result;
 
 	CALL_BACKEND (gnome_vfs_async_write,
@@ -625,7 +626,6 @@ void
 gnome_vfs_async_get_file_info  (GnomeVFSAsyncHandle **handle_return,
 				GList *uris,
 				GnomeVFSFileInfoOptions options,
-				const char * const meta_keys[],
 				GnomeVFSAsyncGetFileInfoCallback callback,
 				gpointer callback_data)
 {
@@ -633,17 +633,74 @@ gnome_vfs_async_get_file_info  (GnomeVFSAsyncHandle **handle_return,
 		(*real_gnome_vfs_async_get_file_info) (GnomeVFSAsyncHandle **handle_return,
 						       GList *uris,
 						       GnomeVFSFileInfoOptions options,
-						       const char * const meta_keys[],
 						       GnomeVFSAsyncGetFileInfoCallback callback,
-						       gpointer callback_data) = NULL;
+						       gpointer callback_data);
 	GnomeVFSResult result;
 
 	CALL_BACKEND (gnome_vfs_async_get_file_info,
-		      (handle_return, uris, options, meta_keys,
+		      (handle_return, uris, options,
 		       callback, callback_data));
 	report_failure_get_file_info (result, uris, callback, callback_data);
 }
 
+static gboolean
+report_failure_set_file_info_callback (gpointer callback_data)
+{
+	CallbackData *data;
+
+	data = callback_data;
+	(* data->callback.set_file_info) (NULL,
+				 data->result,
+				 NULL,
+				 data->callback_data);
+	g_free (data);
+
+	return FALSE;
+}
+
+static void
+report_failure_set_file_info (GnomeVFSResult result,
+		       	      GnomeVFSAsyncSetFileInfoCallback callback,
+		       	      gpointer callback_data)
+{
+	CallbackData *data;
+
+	if (result == GNOME_VFS_OK)
+		return;
+
+	data = g_new0 (CallbackData, 1);
+	data->callback.set_file_info = callback;
+	data->result = result;
+	data->callback_data = callback_data;
+	g_idle_add (report_failure_set_file_info_callback, data);
+}
+
+void
+gnome_vfs_async_set_file_info  (GnomeVFSAsyncHandle **handle_return,
+				GnomeVFSURI *uri,
+				GnomeVFSFileInfo *info,
+				GnomeVFSSetFileInfoMask mask,
+			        GnomeVFSFileInfoOptions options,
+				GnomeVFSAsyncSetFileInfoCallback callback,
+				gpointer callback_data)
+{
+	static GnomeVFSResult	 
+		(*real_gnome_vfs_async_set_file_info) (GnomeVFSAsyncHandle **handle_return,
+						       GnomeVFSURI *uri,
+						       GnomeVFSFileInfo *info,
+						       GnomeVFSSetFileInfoMask mask,
+						       GnomeVFSFileInfoOptions options,
+						       GnomeVFSAsyncSetFileInfoCallback callback,
+						       gpointer callback_data);
+
+	GnomeVFSResult result;
+
+	CALL_BACKEND (gnome_vfs_async_set_file_info,
+		      (handle_return, uri, info, mask, options,
+		       callback, callback_data));
+	report_failure_set_file_info (result, callback, callback_data);
+}
+
 static gboolean
 report_failure_load_directory_callback (gpointer callback_data)
 {
@@ -681,7 +738,6 @@ void
 gnome_vfs_async_load_directory_uri (GnomeVFSAsyncHandle **handle_return,
 				    GnomeVFSURI *uri,
 				    GnomeVFSFileInfoOptions options,
-				    const gchar *meta_keys[],
 				    GnomeVFSDirectorySortRule sort_rules[],
 				    gboolean reverse_order,
 				    GnomeVFSDirectoryFilterType filter_type,
@@ -695,7 +751,6 @@ gnome_vfs_async_load_directory_uri (GnomeVFSAsyncHandle **handle_return,
 		(*real_gnome_vfs_async_load_directory_uri) (GnomeVFSAsyncHandle **handle_return,
 							    GnomeVFSURI *uri,
 							    GnomeVFSFileInfoOptions options,
-							    const gchar *meta_keys[],
 							    GnomeVFSDirectorySortRule sort_rules[],
 							    gboolean reverse_order,
 							    GnomeVFSDirectoryFilterType filter_type,
@@ -703,11 +758,11 @@ gnome_vfs_async_load_directory_uri (GnomeVFSAsyncHandle **handle_return,
 							    const gchar *filter_pattern,
 							    guint items_per_notification,
 							    GnomeVFSAsyncDirectoryLoadCallback callback,
-							    gpointer callback_data) = NULL;
+							    gpointer callback_data);
 	GnomeVFSResult result;
 
 	CALL_BACKEND (gnome_vfs_async_load_directory_uri,
-		      (handle_return, uri, options, meta_keys, sort_rules,
+		      (handle_return, uri, options, sort_rules,
 		       reverse_order, filter_type, filter_options,
 		       filter_pattern, items_per_notification,
 		       callback, callback_data));
@@ -718,7 +773,6 @@ void
 gnome_vfs_async_load_directory (GnomeVFSAsyncHandle **handle_return,
 				const gchar *uri,
 				GnomeVFSFileInfoOptions options,
-				const gchar * const meta_keys[],
 				GnomeVFSDirectorySortRule sort_rules[],
 				gboolean reverse_order,
 				GnomeVFSDirectoryFilterType filter_type,
@@ -732,7 +786,6 @@ gnome_vfs_async_load_directory (GnomeVFSAsyncHandle **handle_return,
 		(*real_gnome_vfs_async_load_directory) (GnomeVFSAsyncHandle **handle_return,
 							const gchar *uri,
 							GnomeVFSFileInfoOptions options,
-							const gchar * const meta_keys[],
 							GnomeVFSDirectorySortRule sort_rules[],
 							gboolean reverse_order,
 							GnomeVFSDirectoryFilterType filter_type,
@@ -740,11 +793,11 @@ gnome_vfs_async_load_directory (GnomeVFSAsyncHandle **handle_return,
 							const gchar *filter_pattern,
 							guint items_per_notification,
 							GnomeVFSAsyncDirectoryLoadCallback callback,
-							gpointer callback_data) = NULL;
+							gpointer callback_data);
 	GnomeVFSResult result;
 
 	CALL_BACKEND (gnome_vfs_async_load_directory,
-		      (handle_return, uri, options, meta_keys, sort_rules, reverse_order,
+		      (handle_return, uri, options, sort_rules, reverse_order,
 		       filter_type, filter_options, filter_pattern, items_per_notification,
 		       callback, callback_data));
 	report_failure_load_directory (result, callback, callback_data);
@@ -776,7 +829,7 @@ gnome_vfs_async_xfer (GnomeVFSAsyncHandle **handle_return,
 					      GnomeVFSAsyncXferProgressCallback progress_update_callback,
 					      gpointer update_callback_data,
 					      GnomeVFSXferProgressCallback progress_sync_callback,
-					      gpointer sync_callback_data) = NULL;
+					      gpointer sync_callback_data);
 
 	CALL_BACKEND_RETURN (gnome_vfs_async_xfer,
 			     (handle_return,
@@ -791,7 +844,7 @@ GnomeVFSResult
 gnome_vfs_async_cancel (GnomeVFSAsyncHandle *handle)
 {
 	static GnomeVFSResult
-		(*real_gnome_vfs_async_cancel)(GnomeVFSAsyncHandle *handle) = NULL;
+		(*real_gnome_vfs_async_cancel)(GnomeVFSAsyncHandle *handle);
 
 	CALL_BACKEND_RETURN (gnome_vfs_async_cancel, (handle));
 }
@@ -804,7 +857,7 @@ gnome_vfs_async_add_status_callback (GnomeVFSAsyncHandle *handle,
 	static guint
 		(*real_gnome_vfs_async_add_status_callback) (GnomeVFSAsyncHandle *handle,
 							     GnomeVFSStatusCallback callback,
-							     gpointer user_data) = NULL;
+							     gpointer user_data);
 
 	CALL_BACKEND_RETURN (gnome_vfs_async_add_status_callback,
 			     (handle, callback, user_data));
@@ -816,14 +869,15 @@ gnome_vfs_async_remove_status_callback (GnomeVFSAsyncHandle *handle,
 {
 	static void
 		(*real_gnome_vfs_async_remove_status_callback) (GnomeVFSAsyncHandle *handle,
-								guint callback_id) = NULL;
+								guint callback_id);
 
-	if (!real_gnome_vfs_async_remove_status_callback)
-	{
-		real_gnome_vfs_async_remove_status_callback = (void (*)())
-			func_lookup("gnome_vfs_async_remove_status_callback");
-		if (!real_gnome_vfs_async_remove_status_callback)
+	if (real_gnome_vfs_async_remove_status_callback == NULL) {
+		real_gnome_vfs_async_remove_status_callback = (void (*) (GnomeVFSAsyncHandle *, guint))
+			func_lookup ("gnome_vfs_async_remove_status_callback");
+		if (real_gnome_vfs_async_remove_status_callback == NULL) {
+			/* Silently do nothing. */
 			return;
+		}
 	}
 
 	real_gnome_vfs_async_remove_status_callback (handle, callback_id);
