@@ -97,7 +97,6 @@ typedef GnomeVFSResult (* GnomeVFSMethodOpenDirectoryFunc)
 					 GnomeVFSMethodHandle **method_handle,
 					 GnomeVFSURI *uri,
 					 GnomeVFSFileInfoOptions options,
-					 const GList *meta_keys,
 					 const GnomeVFSDirectoryFilter *filter,
 					 GnomeVFSContext *context);
 
@@ -117,7 +116,6 @@ typedef GnomeVFSResult (* GnomeVFSMethodGetFileInfoFunc)
 					 GnomeVFSURI *uri,
 					 GnomeVFSFileInfo *file_info,
 					 GnomeVFSFileInfoOptions options,
-					 const GList *meta_keys,
 					 GnomeVFSContext *context);
 
 typedef GnomeVFSResult (* GnomeVFSMethodGetFileInfoFromHandleFunc)
@@ -125,7 +123,6 @@ typedef GnomeVFSResult (* GnomeVFSMethodGetFileInfoFromHandleFunc)
 					 GnomeVFSMethodHandle *method_handle,
 					 GnomeVFSFileInfo *file_info,
 					 GnomeVFSFileInfoOptions options,
-					 const GList *meta_keys,
 					 GnomeVFSContext *context);
 
 typedef GnomeVFSResult (* GnomeVFSMethodTruncateFunc) (GnomeVFSMethod *method,
@@ -187,7 +184,7 @@ typedef GnomeVFSResult (* GnomeVFSMethodSetFileInfo)
 					 GnomeVFSSetFileInfoMask mask,
 					 GnomeVFSContext *context);
 
-typedef GnomeVFSResult (* GnomeVFSMethodCreateSymbolicLink)
+typedef GnomeVFSResult (* GnomeVFSMethodCreateSymbolicLinkFunc)
                                         (GnomeVFSMethod *method,
                                          GnomeVFSURI *uri,
                                          const gchar *target_reference,
@@ -219,7 +216,7 @@ struct GnomeVFSMethod {
 	GnomeVFSMethodSetFileInfo set_file_info;
 	GnomeVFSMethodTruncateFunc truncate;
 	GnomeVFSMethodFindDirectoryFunc find_directory;
-	GnomeVFSMethodCreateSymbolicLink create_symbolic_link;
+	GnomeVFSMethodCreateSymbolicLinkFunc create_symbolic_link;
 };
 
 
