@@ -43,6 +43,7 @@
 #endif
 
 #include <sys/time.h>
+#include <string.h>
 
 #include "gnome-vfs.h"
 #include "gnome-vfs-private.h"
@@ -411,8 +412,13 @@ remove_file (GnomeVFSURI *uri,
 		if (result != GNOME_VFS_OK)
 			retry = handle_error (&result, progress,
 					      error_mode, skip);
-		else
+		else {
 			progress->progress_info->file_index++;
+			/* add some small size for each deleted item because delete overhead
+			 * does not depend on file/directory size
+			 */
+			progress->progress_info->bytes_copied += DEFAULT_SIZE_OVERHEAD;
+		}
 
 		if (call_progress_with_uris_often (progress, uri, NULL, 
 						   GNOME_VFS_XFER_PHASE_DELETESOURCE) 
@@ -437,7 +443,8 @@ empty_directory (GnomeVFSURI *uri,
 	GnomeVFSDirectoryHandle *directory_handle;
 
 	result = gnome_vfs_directory_open_from_uri (&directory_handle, uri, 
-		GNOME_VFS_FILE_INFO_DEFAULT, NULL, NULL);
+						    GNOME_VFS_FILE_INFO_DEFAULT, 
+						    NULL);
 
 	if (result != GNOME_VFS_OK)
 		return result;
@@ -511,9 +518,12 @@ remove_directory (GnomeVFSURI *uri,
 			if (result != GNOME_VFS_OK)
 				retry = handle_error (&result, progress,
 						      error_mode, skip);
-			else
+			else {
 				progress->progress_info->file_index++;
-
+				/* add some small size for each deleted item */
+				progress->progress_info->bytes_total += DEFAULT_SIZE_OVERHEAD;
+				progress->progress_info->total_bytes_copied += DEFAULT_SIZE_OVERHEAD;
+			}
 			if (call_progress_with_uris_often (progress, uri, NULL, 
 							   GNOME_VFS_XFER_PHASE_DELETESOURCE) 
 				== GNOME_VFS_XFER_OVERWRITE_ACTION_ABORT) {
@@ -552,7 +562,7 @@ gnome_vfs_visit_list (const GnomeVFSURI *dir_uri,
 		/* get the URI and VFSFileInfo for each */
 		uri = gnome_vfs_uri_append_file_name (dir_uri, p->data);
 		gnome_vfs_file_info_init (&info);
-		result = gnome_vfs_get_file_info_uri (uri, &info, info_options, NULL);
+		result = gnome_vfs_get_file_info_uri (uri, &info, info_options);
 		
 		if (result == GNOME_VFS_OK) {
 			gboolean tmp_recurse;
@@ -570,7 +580,7 @@ gnome_vfs_visit_list (const GnomeVFSURI *dir_uri,
 				 * recursively 
 				 */
 				result = gnome_vfs_directory_visit_uri
-						(uri, info_options, NULL, NULL, visit_options,
+						(uri, info_options, NULL, visit_options,
 						 callback, data);
 			}
 		}
@@ -615,6 +625,7 @@ count_each_file_size_one (const gchar *rel_path,
 	} else if (info->type == GNOME_VFS_FILE_TYPE_DIRECTORY) {
 		/* add some small size for each directory */
 		params->progress->progress_info->bytes_total += DEFAULT_SIZE_OVERHEAD;
+		params->progress->progress_info->total_bytes_copied += DEFAULT_SIZE_OVERHEAD;
 	}
 
 	/* watch out for infinite recursion*/
@@ -636,7 +647,8 @@ count_items_and_size (const GnomeVFSURI *dir_uri,
 		      const GList *name_list,
 		      GnomeVFSXferOptions xfer_options,
 		      GnomeVFSProgressCallbackState *progress,
-		      gboolean move)
+		      gboolean move,
+		      gboolean link)
 {
 	/*
 	 * FIXME bugzilla.eazel.com 1200:
@@ -664,7 +676,7 @@ count_items_and_size (const GnomeVFSURI *dir_uri,
 	}
 
 	return gnome_vfs_visit_list (dir_uri, name_list, info_options,
-		visit_options, !move && (xfer_options & GNOME_VFS_XFER_RECURSIVE) != 0,
+		visit_options, !link && !move && (xfer_options & GNOME_VFS_XFER_RECURSIVE) != 0,
 		count_each_file_size_one, &each_params);
 }
 
@@ -694,7 +706,7 @@ directory_add_items_and_size (GnomeVFSURI *dir_uri,
 		info_options = GNOME_VFS_FILE_INFO_DEFAULT;
 	}
 
-	return gnome_vfs_directory_visit_uri (dir_uri, info_options, NULL, NULL, 
+	return gnome_vfs_directory_visit_uri (dir_uri, info_options, NULL, 
 		visit_options, count_each_file_size_one, &each_params);
 
 }
@@ -733,7 +745,8 @@ handle_name_conflicts (const GnomeVFSURI *source_dir_uri,
 		/* get the URI and VFSFileInfo for each */
 		uri = gnome_vfs_uri_append_file_name (target_dir_uri, target_item->data);
 		gnome_vfs_file_info_init (&info);
-		result = gnome_vfs_get_file_info_uri (uri, &info, GNOME_VFS_FILE_INFO_DEFAULT, NULL);
+		result = gnome_vfs_get_file_info_uri (uri, &info, 
+						      GNOME_VFS_FILE_INFO_DEFAULT);
 		
 		if (result == GNOME_VFS_OK) {
 			conflict_count++;
@@ -765,7 +778,8 @@ handle_name_conflicts (const GnomeVFSURI *source_dir_uri,
 		/* get the URI and VFSFileInfo for each */
 		uri = gnome_vfs_uri_append_file_name (target_dir_uri, target_item->data);
 		gnome_vfs_file_info_init (&info);
-		result = gnome_vfs_get_file_info_uri (uri, &info, GNOME_VFS_FILE_INFO_DEFAULT, NULL);
+		result = gnome_vfs_get_file_info_uri (uri, &info, 
+						      GNOME_VFS_FILE_INFO_DEFAULT);
 		
 		if (result == GNOME_VFS_OK) {
 			/* FIXME bugzilla.eazel.com 1201:
@@ -872,8 +886,8 @@ create_directory (GnomeVFSURI *dir_uri,
 			}
 			if (force_replace) {
 				result = remove_directory (dir_uri, TRUE, progress, 
-							xfer_options, error_mode, 
-							skip);
+							   xfer_options, error_mode, 
+							   skip);
 			} else {
 				result = GNOME_VFS_OK;
 			}
@@ -883,7 +897,7 @@ create_directory (GnomeVFSURI *dir_uri,
 			return gnome_vfs_directory_open_from_uri (return_handle, 
 								  dir_uri, 
 								  GNOME_VFS_FILE_INFO_DEFAULT, 
-								  NULL, NULL);
+								  NULL);
 		}
 		/* handle the error case */
 		retry = handle_error (&result, progress,
@@ -913,13 +927,12 @@ copy_file_data (GnomeVFSHandle *target_handle,
 
 	*skip = FALSE;
 
-	buffer = g_malloc (block_size);
-
 	if (call_progress_often (progress, GNOME_VFS_XFER_PHASE_COPYING) == 0) {
-		g_free(buffer);
 		return GNOME_VFS_ERROR_INTERRUPTED;
 	}
 
+	buffer = g_malloc (block_size);
+
 	do {
 		GnomeVFSFileSize bytes_read;
 		GnomeVFSFileSize bytes_to_write;
@@ -1110,9 +1123,12 @@ copy_file (GnomeVFSFileInfo *info,
 
 		result = copy_file_data (target_handle, source_handle,
 					progress, xfer_options, error_mode,
-					/* use an arbitrary default block size of 4096 if one isn't available for this fs */
-					info->valid_fields&GNOME_VFS_FILE_INFO_FIELDS_IO_BLOCK_SIZE?info->io_block_size:4096,
-					 skip);
+					/* use an arbitrary default block size of 4096 
+					 * if one isn't available for this file system 
+					 */
+					(info->valid_fields & GNOME_VFS_FILE_INFO_FIELDS_IO_BLOCK_SIZE)
+						? info->io_block_size : 4096,
+					skip);
 	}
 
 	progress->progress_info->file_index++;
@@ -1146,7 +1162,7 @@ copy_directory (GnomeVFSURI *source_dir_uri,
 	dest_directory_handle = NULL;
 	
 	result = gnome_vfs_directory_open_from_uri (&source_directory_handle, source_dir_uri, 
-			GNOME_VFS_FILE_INFO_DEFAULT, NULL, NULL);
+						    GNOME_VFS_FILE_INFO_DEFAULT, NULL);
 
 	if (result != GNOME_VFS_OK) {
 		return result;
@@ -1208,8 +1224,9 @@ copy_directory (GnomeVFSURI *source_dir_uri,
 					result = copy_directory (source_uri, dest_uri, 
 							    xfer_options, error_mode, overwrite_mode, 
 							    progress, skip);
+				} else if (info.type == GNOME_VFS_FILE_TYPE_SYMBOLIC_LINK) {
+					result = gnome_vfs_create_symbolic_link (dest_uri, info.symlink_name);
 				} else {
-					/* need to implement symbolic links*/
 					g_assert (!"unimplemented");
 				}
 
@@ -1268,7 +1285,7 @@ copy_items (const GnomeVFSURI *source_dir_uri,
 		source_uri = gnome_vfs_uri_append_file_name (source_dir_uri, source_name_item->data);
 		gnome_vfs_file_info_init (&info);
 		result = gnome_vfs_get_file_info_uri (source_uri, &info, 
-						      GNOME_VFS_FILE_INFO_DEFAULT, NULL);
+						      GNOME_VFS_FILE_INFO_DEFAULT);
 
 		progress->progress_info->duplicate_name = g_strdup (dest_name_item->data);
 
@@ -1302,8 +1319,9 @@ copy_items (const GnomeVFSURI *source_dir_uri,
 							    xfer_options, error_mode,
 							    &overwrite_mode_abort,
 							    progress, &skip);
-				} else {
-					/* need to implement symbolic links*/
+                                } else if (info.type == GNOME_VFS_FILE_TYPE_SYMBOLIC_LINK) {
+					result = gnome_vfs_create_symbolic_link (target_uri, info.symlink_name);
+                                } else {
 					g_assert (!"unimplemented");
 				}
 
@@ -1456,6 +1474,110 @@ move_items (const GnomeVFSURI *source_dir_uri,
 
 	return result;
 }
+static GnomeVFSResult
+link_items (const GnomeVFSURI *source_dir_uri,
+	    const GList *source_name_list,
+	    GnomeVFSURI *target_dir_uri,
+	    const GList *target_name_list,
+	    GnomeVFSXferOptions xfer_options,
+	    GnomeVFSXferErrorMode *error_mode,
+	    GnomeVFSXferOverwriteMode *overwrite_mode,
+	    GnomeVFSProgressCallbackState *progress)
+{
+	GnomeVFSResult result;
+	const GList *source_name_item;
+	const GList *dest_name_item;
+	GnomeVFSURI *source_uri;
+	GnomeVFSURI *target_uri;
+	gboolean retry;
+	gboolean skip;
+	int conflict_count;
+	int progress_result;
+	char *source_reference;
+
+	result = GNOME_VFS_OK;
+
+	/* go through the list of names, move each item */
+	for (source_name_item = source_name_list, dest_name_item = target_name_list; 
+		source_name_item != NULL;) {
+
+		source_uri = gnome_vfs_uri_append_file_name (source_dir_uri, source_name_item->data);
+		source_reference = gnome_vfs_uri_to_string (source_uri, GNOME_VFS_URI_HIDE_NONE);
+
+		progress->progress_info->duplicate_name = g_strdup (dest_name_item->data);
+
+		retry = FALSE;
+		skip = FALSE;
+		conflict_count = 1;
+
+		do {
+			target_uri = gnome_vfs_uri_append_file_name
+				(target_dir_uri, 
+				 progress->progress_info->duplicate_name);
+
+			progress->progress_info->file_size = DEFAULT_SIZE_OVERHEAD;
+			progress->progress_info->bytes_copied = 0;
+			progress_set_source_target_uris (progress, source_uri, target_uri);
+
+			/* no matter what the replace mode, just overwrite the destination
+			 * handle_name_conflicts took care of conflicting files
+			 */
+			result = gnome_vfs_create_symbolic_link (target_uri, source_reference); 
+
+			if (result == GNOME_VFS_ERROR_FILE_EXISTS) {
+				/* deal with a name conflict -- ask the progress_callback for a better name */
+				g_free (progress->progress_info->duplicate_name);
+				progress->progress_info->duplicate_name = g_strdup (dest_name_item->data);
+				progress->progress_info->duplicate_count = conflict_count;
+				progress->progress_info->status = GNOME_VFS_XFER_PROGRESS_STATUS_DUPLICATE;
+				progress->progress_info->vfs_status = result;
+				progress_result = call_progress_uri (progress, source_uri, target_uri, 
+						       GNOME_VFS_XFER_PHASE_COPYING);
+				progress->progress_info->status = GNOME_VFS_XFER_PROGRESS_STATUS_OK;
+
+				if (progress_result == GNOME_VFS_XFER_OVERWRITE_ACTION_ABORT) {
+					gnome_vfs_uri_unref (target_uri);
+					break;
+				}
+				conflict_count++;
+				
+				result = gnome_vfs_create_symbolic_link (target_uri, source_reference);
+
+			}
+			
+			if (result != GNOME_VFS_OK) {
+				if (result == GNOME_VFS_ERROR_FILE_EXISTS) {
+					retry = TRUE;
+				} else {
+				retry = handle_error (&result, progress,
+						      error_mode, &skip);
+				}
+			} else {
+				retry = FALSE;
+			}
+
+			if (call_progress_with_uris_often (progress, source_uri,
+						target_uri, GNOME_VFS_XFER_PHASE_OPENTARGET) == 0) {
+				result = GNOME_VFS_ERROR_INTERRUPTED;
+				gnome_vfs_uri_unref (target_uri);
+				break;
+			}
+			gnome_vfs_uri_unref (target_uri);
+		} while (retry);
+		
+		gnome_vfs_uri_unref (source_uri);
+		g_free (source_reference);
+
+		if (result != GNOME_VFS_OK && !skip)
+			break;
+
+		source_name_item = source_name_item->next;
+		dest_name_item = dest_name_item->next;
+		g_assert ((source_name_item != NULL) == (dest_name_item != NULL));
+	}
+
+	return result;
+}
 
 static GnomeVFSResult
 gnome_vfs_xfer_empty_trash (GList *trash_dir_uris,
@@ -1482,6 +1604,11 @@ gnome_vfs_xfer_empty_trash (GList *trash_dir_uris,
 			progress);
 		if (result != GNOME_VFS_OK)
 			break;
+		/* set up a fake total size to represent the bulk of the operation
+		 * -- we'll subtract a proportional value for every deletion
+		 */
+		progress->progress_info->bytes_total 
+			= progress->progress_info->files_total * DEFAULT_SIZE_OVERHEAD;
 	}
 	if (result == GNOME_VFS_OK) {
 		call_progress (progress, GNOME_VFS_XFER_PHASE_READYTOGO);
@@ -1500,11 +1627,11 @@ gnome_vfs_xfer_empty_trash (GList *trash_dir_uris,
 }
 
 static GnomeVFSResult
-gnome_vfs_xfer_delete_items (GnomeVFSURI *source_directory,
-			     const GList *item_names,
-			     GnomeVFSXferErrorMode error_mode,
-			     GnomeVFSXferOptions xfer_options,
-			     GnomeVFSProgressCallbackState *progress)
+gnome_vfs_xfer_delete_items_common (GnomeVFSURI *source_directory,
+			            const GList *item_names,
+			            GnomeVFSXferErrorMode error_mode,
+			            GnomeVFSXferOptions xfer_options,
+			            GnomeVFSProgressCallbackState *progress)
 {
 	GnomeVFSFileInfo info;
 	GnomeVFSResult result;
@@ -1514,6 +1641,46 @@ gnome_vfs_xfer_delete_items (GnomeVFSURI *source_directory,
 
 	result = GNOME_VFS_OK;
 	
+	for (p = item_names;  p != NULL; p = p->next) {
+	
+		skip = FALSE;
+		/* get the URI and VFSFileInfo for each */
+		uri = gnome_vfs_uri_append_file_name (source_directory, p->data);
+
+		gnome_vfs_file_info_init (&info);
+		result = gnome_vfs_get_file_info_uri (uri, &info, 
+						      GNOME_VFS_FILE_INFO_DEFAULT);
+
+		if (result == GNOME_VFS_OK) {
+			if (info.type == GNOME_VFS_FILE_TYPE_DIRECTORY) {
+				remove_directory (uri, TRUE, progress, xfer_options, 
+						  &error_mode, &skip);
+			} else {
+				remove_file (uri, progress, xfer_options, &error_mode,
+					     &skip);
+			}
+		}
+		gnome_vfs_uri_unref (uri);
+		if (result != GNOME_VFS_OK) {
+			break;
+		}
+	}
+
+	return result;
+}
+
+static GnomeVFSResult
+gnome_vfs_xfer_delete_items (GnomeVFSURI *source_directory,
+			     const GList *item_names,
+			     GnomeVFSXferErrorMode error_mode,
+			     GnomeVFSXferOptions xfer_options,
+			     GnomeVFSProgressCallbackState *progress)
+{
+	GnomeVFSResult result;
+	const GList *p;
+
+	result = GNOME_VFS_OK;
+	
 	call_progress (progress, GNOME_VFS_XFER_PHASE_INITIAL);
 
 		/* initialize the results */
@@ -1524,37 +1691,21 @@ gnome_vfs_xfer_delete_items (GnomeVFSURI *source_directory,
 
 	for (p = item_names;  p != NULL; p = p->next) {
 		result = count_items_and_size (source_directory, item_names,
-			GNOME_VFS_XFER_REMOVESOURCE | GNOME_VFS_XFER_RECURSIVE, 
-			progress, TRUE);
+			GNOME_VFS_XFER_REMOVESOURCE | GNOME_VFS_XFER_RECURSIVE, 			progress, TRUE, FALSE);
 		if (result != GNOME_VFS_OK)
 			break;
+
+		/* set up a fake total size to represent the bulk of the operation
+		 * -- we'll subtract a proportional value for every deletion
+		 */
+		progress->progress_info->bytes_total 
+			= progress->progress_info->files_total * DEFAULT_SIZE_OVERHEAD;
 	}
 
 	if (result == GNOME_VFS_OK) {
 		call_progress (progress, GNOME_VFS_XFER_PHASE_READYTOGO);
-		for (p = item_names;  p != NULL; p = p->next) {
-		
-			skip = FALSE;
-			/* get the URI and VFSFileInfo for each */
-			uri = gnome_vfs_uri_append_file_name (source_directory, p->data);
-
-			gnome_vfs_file_info_init (&info);
-			result = gnome_vfs_get_file_info_uri (uri, &info, GNOME_VFS_FILE_INFO_DEFAULT, NULL);
-
-			if (result == GNOME_VFS_OK) {
-				if (info.type == GNOME_VFS_FILE_TYPE_DIRECTORY) {
-					remove_directory (uri, TRUE, progress, xfer_options, 
-							  &error_mode, &skip);
-				} else {
-					remove_file (uri, progress, xfer_options, &error_mode,
-						     &skip);
-				}
-			}
-			gnome_vfs_uri_unref (uri);
-			if (result != GNOME_VFS_OK) {
-				break;
-			}
-		}
+		result = gnome_vfs_xfer_delete_items_common (source_directory, item_names,
+			error_mode, xfer_options, progress);
 	}
 
 	call_progress (progress, GNOME_VFS_XFER_PHASE_COMPLETED);
@@ -1594,7 +1745,8 @@ gnome_vfs_new_directory_with_unique_name (const char *target_dir,
 					   progress,
 					   &dummy);
 
-		if (result != GNOME_VFS_ERROR_FILE_EXISTS) {
+		if (result != GNOME_VFS_ERROR_FILE_EXISTS
+			&& result != GNOME_VFS_ERROR_NAME_TOO_LONG) {
 			break;
 		}
 
@@ -1641,10 +1793,11 @@ gnome_vfs_xfer_uri_internal (GnomeVFSURI *source_dir_uri,
 	GnomeVFSResult result;
 	GList *source_name_list;
 	GList *target_name_list;
-	gboolean move;
+	gboolean move, link;
 
 	result = GNOME_VFS_OK;
 	move = FALSE;
+	link = FALSE;
 
 	/* Create an owning list of source and destination names.
 	 * We want to be able to remove items that we decide to skip during
@@ -1660,6 +1813,7 @@ gnome_vfs_xfer_uri_internal (GnomeVFSURI *source_dir_uri,
 	 * and bail if not
 	 */
 	move = (xfer_options & GNOME_VFS_XFER_REMOVESOURCE) != 0;
+	link = (xfer_options & GNOME_VFS_XFER_LINK_ITEMS) != 0;
 
 	if ((xfer_options & GNOME_VFS_XFER_USE_UNIQUE_NAMES) == 0) {
 		gboolean same_fs;
@@ -1674,7 +1828,7 @@ gnome_vfs_xfer_uri_internal (GnomeVFSURI *source_dir_uri,
 
 		progress->progress_info->phase = GNOME_VFS_XFER_PHASE_COLLECTING;
 		result = count_items_and_size (source_dir_uri, source_name_list,
-					       xfer_options, progress, move);
+					       xfer_options, progress, move, link);
 		if (result == GNOME_VFS_OK) {
 
 			/* FIXME bugzilla.eazel.com 1214:
@@ -1703,10 +1857,16 @@ gnome_vfs_xfer_uri_internal (GnomeVFSURI *source_dir_uri,
 				call_progress (progress, GNOME_VFS_XFER_PHASE_READYTOGO);
 
 				if (move) {
+					g_assert (!link);
 					result = move_items (source_dir_uri, source_name_list,
 							     target_dir_uri, target_name_list,
 							     xfer_options, &error_mode, &overwrite_mode, 
 							     progress);
+				} else if (link) {
+					result = link_items (source_dir_uri, source_name_list,
+							     target_dir_uri, target_name_list,
+							     xfer_options, &error_mode, &overwrite_mode,
+							     progress);
 				} else {
 					result = copy_items (source_dir_uri, source_name_list,
 							     target_dir_uri, target_name_list,
@@ -1715,10 +1875,10 @@ gnome_vfs_xfer_uri_internal (GnomeVFSURI *source_dir_uri,
 				}
 				
 				if (result == GNOME_VFS_OK) {
-
-					if (!move && (xfer_options & GNOME_VFS_XFER_REMOVESOURCE)) {
-						/* FIXME bugzilla.eazel.com 1215:
-						 */
+					if (xfer_options & GNOME_VFS_XFER_REMOVESOURCE) {
+						call_progress (progress, GNOME_VFS_XFER_PHASE_CLEANUP);
+						result = gnome_vfs_xfer_delete_items_common (source_dir_uri,
+							source_names, error_mode, xfer_options, progress);
 					}
 				}
 			}
